﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Security.Cryptography;
using PasswordCrackerCentralized.model;
using PasswordCrackerCentralized.util;

namespace PasswordCrackerCentralized
{
    public static class CrackHelpers
    {
        public static IEnumerable<String> Variations(this IEnumerable<String> dictionaryEntries)
        {
            if (dictionaryEntries == null) throw new ArgumentNullException("dictionaryEntries");
            foreach (var dictionaryEntry in dictionaryEntries)
            {
                yield return dictionaryEntry;
                yield return dictionaryEntry.ToUpper();
                yield return StringUtilities.Capitalize(dictionaryEntry);
                yield return StringUtilities.Reverse(dictionaryEntry);
                for (int i = 0; i < 100; i++)
                {
                    yield return i + dictionaryEntry;
                    yield return dictionaryEntry + i;
                }
                for (int i = 0; i < 10; i++)
                {
                    for (int j = 0; j < 10; j++)
                    {
                        yield return i + dictionaryEntry + j;
                    }
                }
            }
        }

        public static IEnumerable<Tuple<String, byte[]>> Encrypt(this IEnumerable<String> variations)
        {
            HashAlgorithm messageDigest = new SHA1CryptoServiceProvider();
            foreach (var variation in variations)
            {
                char[] charArray = variation.ToCharArray();
                byte[] passwordAsBytes = Array.ConvertAll(charArray, PasswordFileHandler.GetConverter());
                yield return new Tuple<string, byte[]>(variation, messageDigest.ComputeHash(passwordAsBytes));
            }
        }

        public static IEnumerable<Tuple<String, byte[]>> Encrypt(this Para llelQuery<String> variations)
        {
            HashAlgorithm messageDigest = new SHA1CryptoServiceProvider();
            //foreach (var variation in variations)
            variations.ForAll(variation =>
            {
                char[] charArray = variation.ToCharArray();
                byte[] passwordAsBytes = Array.ConvertAll(charArray, PasswordFileHandler.GetConverter());
                yield return new Tuple<string, byte[]>(variation, messageDigest.ComputeHash(passwordAsBytes));
            });
        }

        public static IEnumerable<UserInfoClearText> Check(this IEnumerable<Tuple<String, byte[]>> tuples, IList<UserInfo> userInfos)
        {
            foreach (var tuple in tuples)
            {
                foreach (var userInfo in userInfos)
                {
                    if (tuple.Item2.SequenceEqual(userInfo.EntryptedPassword))
                    {
                        // Future improvement: userInfos remove
                        Console.WriteLine(userInfo.Username + " " + tuple.Item1);
                        yield return new UserInfoClearText(userInfo.Username, tuple.Item1);
                    }
                }
            }
        }
    }
}
